@@ -1170,10 +1170,6 @@ static int if_sdio_probe(struct sdio_func *func,
 
 	spin_lock_init(&card->lock);
 	card->workqueue = alloc_workqueue("libertas_sdio", WQ_MEM_RECLAIM, 0);
-	if (unlikely(!card->workqueue)) {
-		ret = -ENOMEM;
-		goto err_queue;
-	}
 	INIT_WORK(&card->packet_worker, if_sdio_host_to_card_worker);
 	init_waitqueue_head(&card->pwron_waitq);
 
@@ -1225,7 +1221,6 @@ err_activate_card:
 	lbs_remove_card(priv);
 free:
 	destroy_workqueue(card->workqueue);
-err_queue:
 	while (card->packets) {
 		packet = card->packets;
 		card->packets = card->packets->next;
